"""abs_move

迁移 ID: 74381965bbe2
父迁移: 1e67057a1646
创建时间: 2026-01-10 10:29:06.245119

"""

from __future__ import annotations

from collections.abc import Sequence

import sqlalchemy as sa
from alembic import op

revision: str = "74381965bbe2"
down_revision: str | Sequence[str] | None = "1e67057a1646"
branch_labels: str | Sequence[str] | None = None
depends_on: str | Sequence[str] | None = None


def upgrade(name: str = "") -> None:
    if name:
        return
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("suggarchat_memory_data", schema=None) as batch_op:
        batch_op.drop_column("memory_abstract")

    # ### end Alembic commands ###


def downgrade(name: str = "") -> None:
    if name:
        return
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("suggarchat_memory_data", schema=None) as batch_op:
        batch_op.add_column(
            sa.Column(
                "memory_abstract",
                sa.TEXT(),
                server_default=sa.text("('')"),
                nullable=False,
            )
        )

    # ### end Alembic commands ###
