from nonebot import require
from nonebot.plugin import PluginMetadata

require("nonebot_plugin_localstore")
from . import commands, manager
from .models import MatcherData

__all__ = [
    "MatcherData",
    "commands",
    "manager",
]
__plugin_meta__ = PluginMetadata(
    name="Amrita菜单",
    description="菜单功能",
    usage="菜单功能",
)
